package org.netkernel.layer0.nkf.impl;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.SynchronousRequestor;
import org.netkernel.request.*;
import org.netkernel.request.impl.RequestFactory;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.scheduler.RequestFrameException;
import org.netkernel.scheduler.TraceDebugCapture;
import org.netkernel.urii.*;
import org.netkernel.urii.impl.ResolutionImpl;
import org.netkernel.util.ExclusionSet;
import org.netkernel.util.Utils;


public class NKFResolutionContextImpl extends NKFContextImpl implements INKFResolutionContext
{
	private final TraceDebugCapture mDebug;
	private final NKFRequestReadOnlyImpl mRequestToResolve;
	
	public NKFResolutionContextImpl(IKernel aKernel, ISpace aSpace, NKFRequestReadOnlyImpl aRequest, Object aEndpoint)
	{	super(aKernel, aSpace, aRequest, aEndpoint,null);
		IRequest req=aRequest.getKernelRequest();
		mDebug = (TraceDebugCapture)req.getValue(IRequest.TYPE_RESOLVE_DEBUG);
		IRequest toResolve=(IRequest)req.getValue(IRequest.TYPE_RESOLVE_REQUEST);
		mRequestToResolve = new NKFRequestReadOnlyImpl(toResolve);
		mRequestToResolve.setContext(this);
		if (mDebug!=null)
		{	mDebug.push();
		}
		//default response is null - nothing found
	}

	public INKFRequestReadOnly getRequestToResolve()
	{	return mRequestToResolve;
	}

	public void delegateResolution(ISpace aDelegate) throws NKFException
	{	if (!aDelegate.isCommissioned())
		{	NKFException e = createFormattedException("EX_UNCOMMISSIONED_SPACE","MSG_UNCOMMISSIONED_SPACE",null,null,aDelegate);
			throw e;
		}
		
		IRequest thisRequest = getThisKernelRequest();
		ExclusionSet eliminatedSpaces = (ExclusionSet)thisRequest.getValue(IRequest.TYPE_RESOLVE_EXCLUDED_SPACES);
		if (!eliminatedSpaces.contains(aDelegate))
		{	IRequest toResolveIn = (IRequest)thisRequest.getValue(IRequest.TYPE_RESOLVE_REQUEST);
			IRequestScopeLevel rc = RequestScopeLevelImpl.createNonDurableScopeLevel(aDelegate,toResolveIn.getRequestScope());
			IRequest toResolveOut=RequestFactory.cloneWithScope(toResolveIn,rc);
			SynchronousRequestor sr = new SynchronousRequestor(aDelegate);
			IRequest req = RequestFactory.createResolveRequest(toResolveOut,eliminatedSpaces,mDebug,getThisKernelRequest(),sr);
			try
			{	IResponse resp=sr.synchronousRequest(req);
				IResolution resolution = (IResolution)resp.getRepresentation();
				if (resolution!=null)
				{	createResponseFrom(resolution);
				}
			}
			catch (INetKernelException e)
			{	RequestFrameException ex = new RequestFrameException(req,e); // simulate subreq through kernel
				throw new NKFException(ex);
			}
		}
	}
	
	public IResolution getResolution()
	{	IResolution result=null;
		if (mResponse!=null)
		{	result=(IResolution)mResponse.getRepresentation();
		}
		return result;
	}

	public boolean isDebugResolve()
	{	return mDebug!=null;
	}

	public void addResolutionDebug(String aDebug)
	{	addResolutionDebug(aDebug,0,null);
	}
	
	public void addResolutionDebug(String aDebug, int aLevel, Object aEntity)
	{	if (mDebug!=null)
		{	mDebug.add(aDebug,aLevel,aEntity);
		}
	}
	
	public void insertResolutionDebug(String aDebug, int aLevel, Object aEntity)
	{	if (mDebug!=null)
		{	mDebug.insert(aDebug,aLevel,aEntity);
		}
	}

	public void pushResolutionDebug()
	{	if (mDebug!=null)
		{	mDebug.push();
		}
	}
	
	public void popResolutionDebug()
	{	if (mDebug!=null)
		{	mDebug.pop();
		}
	}
	
	public INKFResponse createResolutionTerminateResponse()
	{	return createResolutionResponse(null,null);
	}
	
	public INKFResponse createResolutionResponse(IEndpoint aEndpoint)
	{	return createResolutionResponse(aEndpoint,null);
	}

	public INKFResponse createResolutionResponse(IEndpoint aEndpoint, String aIdentifier)
	{	return createResolutionResponse(aEndpoint,aIdentifier,mRequestToResolve.getKernelRequest().getRequestScope());
	}
	
	public INKFResponse createResolutionResponse(IEndpoint aEndpoint, String aIdentifier, IRequestScopeLevel aRequestScope)
	{	IIdentifier id;
		if(aIdentifier==null)
		{	id=null;			
		}
		else
		{	id= new SimpleIdentifierImpl(aIdentifier);
		}
		IResolution resolution = new ResolutionImpl(aEndpoint,id,aRequestScope);
		return createResponseFrom(resolution);
	}

	public IResolution resolve(INKFRequest aRequest) throws NKFException
	{	IRequest toResolve=((NKFRequestImpl)aRequest).getKernelRequest();
		IRequest req=RequestFactory.createResolveRequest(toResolve, null, null, getKernelContext().getThisKernelRequest(), null);
		IResponse resp=issueKernelRequest(req);
		IResolution result=null;
		if (resp.getRepresentation() instanceof IResolution)
		{	result=(IResolution)resp.getRepresentation();
		}
		return result;
	}

	public boolean canSourcePrimaryAs(Class aRepresentationClass) throws NKFException
	{	boolean result;
		INKFResponseReadOnly primary = getRequestToResolve().getPrimaryAsResponse();
		if (aRepresentationClass.isAssignableFrom(primary.getRepresentation().getClass()))
		{	result=true;
		}
		else
		{	String id=SimpleIdentifierImpl.getUnique("literal:var").toString();
			INKFRequest req=createRequest(id);
			req.setVerb(INKFRequestReadOnly.VERB_TRANSREPRESENT);
			req.addPrimaryArgumentFromResponse(primary);
			req.setRepresentationClass(aRepresentationClass);
			IResolution resolution = resolve(req);
			result = resolution!=null;
		}
		return result;
	}
}